import { SagaIterator } from "@redux-saga/core";
import { FluxStandardAction } from "flux-standard-action";
import { Reducer, ReducersMapObject } from "redux";
export declare type ReadonlyRecord<K extends string, T> = {
    readonly [P in K]: T;
};
export declare type SideEffectRecord<A = any> = ReadonlyRecord<string, (...args: A[]) => any>;
export declare type ActionTypes<A extends SideEffectRecord<any>> = {
    readonly [P in keyof A]: readonly [string, string, string, string];
};
declare type Action = FluxStandardAction<string, {
    readonly result?: unknown;
    readonly error?: unknown;
}>;
declare type SideEffectAction<A extends SideEffectRecord<any>, B extends ActionTypes<A>, P extends keyof A> = {
    readonly type: B[P][0];
    readonly payload: {
        readonly params: Parameters<A[P]>;
    };
};
export declare type ExtractResult<A extends (...a: any[]) => any> = ReturnType<A> extends Promise<infer Result> ? Result : ReturnType<A>;
declare type SagaActionsArray<P extends keyof A, A extends SideEffectRecord<any>, B extends ActionTypes<A>, ErrorType extends unknown = Error> = readonly [(...params: Parameters<A[P]>) => SideEffectAction<A, B, P>, (result: ExtractResult<A[P]>) => {
    readonly type: B[P][1];
    readonly payload: {
        readonly result: ExtractResult<A[P]>;
    };
}, (error: ErrorType) => {
    readonly type: B[P][2];
    readonly payload: {
        readonly error: ErrorType;
    };
    readonly error: true;
}, Reducer<object, Action>, (action: SideEffectAction<A, B, P>) => SagaIterator<void>];
declare type SagaActionsRecord<A extends SideEffectRecord<any>, B extends ActionTypes<A>, ErrorType extends unknown = Error> = {
    readonly [P in keyof A]: SagaActionsArray<P, A, B, ErrorType>;
};
export declare type SagaBoilerplate<A extends SideEffectRecord<any>, B extends ActionTypes<A>, S, ErrorType extends unknown = Error> = {
    readonly actions: SagaActionsRecord<A, B, ErrorType>;
    readonly rootReducer: ReducersMapObject<S>;
    readonly rootSaga: () => SagaIterator<void>;
};
export declare type State<ErrorType extends unknown = Error, ResultType = unknown> = {
    readonly loading: boolean;
    readonly error?: ErrorType;
    readonly result?: ResultType;
};
export declare const concoctBoilerplate: <A extends ReadonlyRecord<string, (...args: any[]) => any>, B extends ActionTypes<A>>(sideEffects: A, actionTypes: B) => SagaBoilerplate<A, B, ReadonlyRecord<string, State<Error, unknown>>, Error>;
export {};
