"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const concoctCrud_1 = require("./concoctCrud");
// tslint:disable: no-expression-statement
it("concocts CRUD boilerplate", async () => {
    const userApi = {
        list: () => {
            return Promise.resolve([
                {
                    id: "1234",
                    username: "username"
                }
            ]);
        },
        get: (id) => {
            return Promise.resolve({
                id,
                username: "username"
            });
        },
        create: (user) => {
            return Promise.resolve({
                id: "1234",
                username: user.username
            });
        },
        update: (id, user) => {
            return Promise.resolve({
                id,
                username: user.username
            });
        },
        delete: (id) => {
            return Promise.resolve(id);
        }
    };
    const { actions: { get: [getUser, getUserSuccess], list: [listUsers] }, rootReducer, rootSaga } = concoctCrud_1.concoctCrud("user", "users", userApi);
    const getAction = getUser("42");
    const listAction = listUsers();
    expect(rootReducer.users).toBeDefined();
    const nextState = rootReducer.users({
        list: { loading: false, result: [{ id: "1234", username: "username" }] },
        current: { loading: false },
        created: { loading: false },
        updated: { loading: false },
        deleted: { loading: false }
    }, getUserSuccess({ id: "5678", username: "anotheruser" }));
    expect(nextState).toEqual({
        list: { loading: false, result: [{ id: "1234", username: "username" }] },
        current: {
            result: {
                id: "5678",
                username: "anotheruser"
            },
            loading: false
        },
        created: { loading: false },
        updated: { loading: false },
        deleted: { loading: false }
    });
    rootSaga();
    expect(getAction).toEqual({
        type: "GET_USER",
        payload: {
            params: ["42"]
        }
    });
    expect(listAction).toEqual({
        type: "LIST_USERS",
        payload: {
            params: []
        }
    });
});
